@@ -1,19 +1,19 @@
-#include <cmus.h>
-#include <lib.h>
-#include <pl.h>
-#include <player.h>
-#include <input.h>
-#include <play_queue.h>
-#include <worker.h>
-#include <track_db.h>
-#include <misc.h>
-#include <file.h>
-#include <utils.h>
-#include <path.h>
-#include <options.h>
-#include <xmalloc.h>
-#include <xstrjoin.h>
-#include <debug.h>
+#include "cmus.h"
+#include "lib.h"
+#include "pl.h"
+#include "player.h"
+#include "input.h"
+#include "play_queue.h"
+#include "worker.h"
+#include "track_db.h"
+#include "misc.h"
+#include "file.h"
+#include "utils.h"
+#include "path.h"
+#include "options.h"
+#include "xmalloc.h"
+#include "xstrjoin.h"
+#include "debug.h"
 #include "load_dir.h"
 
 #include <sys/types.h>
