@@ -3,7 +3,19 @@
 #include "xmalloc.h"
 #include "debug.h"
 
+#include <FLAC/export.h>
+
+#ifdef FLAC_API_VERSION_CURRENT
+/* flac 1.1.3 */
+#define FLAC_NEW_API 1
+#endif
+
+#ifdef FLAC_NEW_API
+#include <FLAC/stream_decoder.h>
+#else
 #include <FLAC/seekable_stream_decoder.h>
+#endif
+
 #include <FLAC/metadata.h>
 #include <inttypes.h>
 #include <sys/types.h>
@@ -17,10 +29,17 @@
 /* Reduce typing.  Namespaces are nice but FLAC API is fscking ridiculous.  */
 
 /* functions, types, enums */
+#ifdef FLAC_NEW_API
+#define F(s) FLAC__stream_decoder_ ## s
+#define T(s) FLAC__StreamDecoder ## s
+#define Dec FLAC__StreamDecoder
+#define E(s) FLAC__STREAM_DECODER_ ## s
+#else
 #define F(s) FLAC__seekable_stream_decoder_ ## s
 #define T(s) FLAC__SeekableStreamDecoder ## s
 #define Dec FLAC__SeekableStreamDecoder
 #define E(s) FLAC__SEEKABLE_STREAM_DECODER_ ## s
+#endif
 
 struct flac_private {
 	/* file/stream position and length */
@@ -51,10 +70,18 @@ static T(ReadStatus) read_cb(const Dec *dec, unsigned char *buf, unsigned *size,
 	if (priv->eof) {
 		*size = 0;
 		d_print("EOF! EOF! EOF!\n");
+#ifdef FLAC_NEW_API
+		return E(READ_STATUS_END_OF_STREAM);
+#else
 		return E(READ_STATUS_OK);
+#endif
 	}
 	if (*size == 0)
+#ifdef FLAC_NEW_API
+		return E(READ_STATUS_CONTINUE);
+#else
 		return E(READ_STATUS_OK);
+#endif
 
 	rc = read(ip_data->fd, buf, *size);
 	if (rc == -1) {
@@ -62,9 +89,17 @@ static T(ReadStatus) read_cb(const Dec *dec, unsigned char *buf, unsigned *size,
 		if (errno == EINTR || errno == EAGAIN) {
 			/* FIXME: not sure how the flac decoder handles this */
 			d_print("interrupted\n");
+#ifdef FLAC_NEW_API
+			return E(READ_STATUS_CONTINUE);
+#else
 			return E(READ_STATUS_OK);
+#endif
 		}
+#ifdef FLAC_NEW_API
+		return E(READ_STATUS_ABORT);
+#else
 		return E(READ_STATUS_ERROR);
+#endif
 	}
 	if (*size != rc) {
 		d_print("%d != %d\n", rc, *size);
@@ -75,9 +110,17 @@ static T(ReadStatus) read_cb(const Dec *dec, unsigned char *buf, unsigned *size,
 		/* never reached! */
 		priv->eof = 1;
 		d_print("EOF\n");
+#ifdef FLAC_NEW_API
+		return E(READ_STATUS_END_OF_STREAM);
+#else
 		return E(READ_STATUS_OK);
+#endif
 	}
+#ifdef FLAC_NEW_API
+	return E(READ_STATUS_CONTINUE);
+#else
 	return E(READ_STATUS_OK);
+#endif
 }
 
 static T(SeekStatus) seek_cb(const Dec *dec, uint64_t offset, void *data)
@@ -375,6 +418,7 @@ static int flac_open(struct input_plugin_data *ip_data)
 	}
 	ip_data->private = priv;
 
+#ifndef FLAC_NEW_API
 	F(set_read_callback)(dec, read_cb);
 	F(set_seek_callback)(dec, seek_cb);
 	F(set_tell_callback)(dec, tell_cb);
@@ -384,11 +428,18 @@ static int flac_open(struct input_plugin_data *ip_data)
 	F(set_metadata_callback)(dec, metadata_cb);
 	F(set_error_callback)(dec, error_cb);
 	F(set_client_data)(dec, ip_data);
+#endif
 
+#ifdef FLAC_NEW_API
+	if (FLAC__stream_decoder_init_stream(dec, read_cb, seek_cb, tell_cb,
+				length_cb, eof_cb, write_cb, metadata_cb,
+				error_cb, ip_data) != E(INIT_STATUS_OK)) {
+#else
 	/* FLAC__METADATA_TYPE_STREAMINFO already accepted */
 	F(set_metadata_respond)(dec, FLAC__METADATA_TYPE_VORBIS_COMMENT);
 
 	if (F(init)(dec) != E(OK)) {
+#endif
 		int save = errno;
 
 		d_print("init failed\n");
@@ -480,9 +531,6 @@ static int flac_seek(struct input_plugin_data *ip_data, double offset)
 	struct flac_private *priv = ip_data->private;
 	uint64_t sample;
 
-	if (ip_data->remote)
-		return -IP_ERROR_ERRNO;
-
 	sample = (uint64_t)(offset * (double)sf_get_rate(ip_data->sf) + 0.5);
 	if (!F(seek_absolute)(priv->dec, sample)) {
 		return -IP_ERROR_ERRNO;
