@@ -17,20 +17,19 @@
  * 02111-1307, USA.
  */
 
-#include <input.h>
-#include <ip.h>
-#include <pcm.h>
-#include <http.h>
-#include <xmalloc.h>
-#include <file.h>
-#include <utils.h>
-#include <cmus.h>
-#include <list.h>
-#include <prog.h>
-#include <misc.h>
-#include <debug.h>
+#include "input.h"
+#include "ip.h"
+#include "pcm.h"
+#include "http.h"
+#include "xmalloc.h"
+#include "file.h"
+#include "utils.h"
+#include "cmus.h"
+#include "list.h"
+#include "prog.h"
+#include "misc.h"
+#include "debug.h"
 #include "config/libdir.h"
-#include "config/version.h"
 
 #include <unistd.h>
 #include <string.h>
@@ -50,6 +49,9 @@ struct input_plugin {
 	int http_code;
 	char *http_reason;
 
+	/* cached duration, -1 = unset */
+	int duration;
+
 	/*
 	 * pcm is converted to 16-bit signed little-endian stereo
 	 * NOTE: no conversion is done if channels > 2 or bits > 16
@@ -401,6 +403,7 @@ static void ip_init(struct input_plugin *ip, char *filename)
 	memset(ip, 0, sizeof(*ip));
 	ip->http_code = -1;
 	ip->pcm_convert_scale = -1;
+	ip->duration = -1;
 	ip->data.fd = -1;
 	ip->data.filename = filename;
 	ip->data.remote = is_url(filename);
@@ -447,10 +450,8 @@ int ip_open(struct input_plugin *ip)
 				rc == -1 ? strerror(errno) : "");
 		if (ip->data.fd != -1)
 			close(ip->data.fd);
-		ip->data.fd = -1;
-		ip->ops = NULL;
 		free(ip->data.metadata);
-		ip->data.metadata = NULL;
+		ip_init(ip, ip->data.filename);
 		return rc;
 	}
 	ip->open = 1;
@@ -507,6 +508,7 @@ int ip_read(struct input_plugin *ip, char *buffer, int count)
 	/* 4608 seems to be optimal for mp3s, 4096 for oggs */
 	char tmp[8 * 1024];
 	char *buf;
+	int sample_size;
 	int rc;
 
 	BUG_ON(count <= 0);
@@ -518,7 +520,6 @@ int ip_read(struct input_plugin *ip, char *buffer, int count)
 	tv.tv_usec = 50e3;
 	rc = select(ip->data.fd + 1, &readfds, NULL, NULL, &tv);
 	if (rc == -1) {
-		d_print("select: error: %s\n", strerror(errno));
 		if (errno == EINTR)
 			errno = EAGAIN;
 		return -1;
@@ -538,21 +539,23 @@ int ip_read(struct input_plugin *ip, char *buffer, int count)
 	}
 
 	rc = ip->ops->read(&ip->data, buf, count);
-	if (rc == 0)
-		ip->eof = 1;
-	if (rc == -1)
-		d_print("error: %s\n", strerror(errno));
-
-	if (rc > 0) {
-		int sample_size = sf_get_sample_size(ip->data.sf);
-
-		if (ip->pcm_convert_in_place != NULL)
-			ip->pcm_convert_in_place(buf, rc / sample_size);
-		if (ip->pcm_convert != NULL)
-			ip->pcm_convert(buffer, tmp, rc / sample_size);
-		rc *= ip->pcm_convert_scale;
+	if (rc == -1 && (errno == EAGAIN || errno == EINTR)) {
+		errno = EAGAIN;
+		return -1;
 	}
-	return rc;
+	if (rc <= 0) {
+		ip->eof = 1;
+		return rc;
+	}
+
+	BUG_ON((rc & ~((unsigned int)sf_get_frame_size(ip->data.sf) - 1U)) != rc);
+
+	sample_size = sf_get_sample_size(ip->data.sf);
+	if (ip->pcm_convert_in_place != NULL)
+		ip->pcm_convert_in_place(buf, rc / sample_size);
+	if (ip->pcm_convert != NULL)
+		ip->pcm_convert(buffer, tmp, rc / sample_size);
+	return rc * ip->pcm_convert_scale;
 }
 
 int ip_seek(struct input_plugin *ip, double offset)
@@ -574,7 +577,13 @@ int ip_read_comments(struct input_plugin *ip, struct keyval **comments)
 
 int ip_duration(struct input_plugin *ip)
 {
-	return ip->ops->duration(&ip->data);
+	if (ip->data.remote)
+		return -1;
+	if (ip->duration == -1)
+		ip->duration = ip->ops->duration(&ip->data);
+	if (ip->duration < 0)
+		return -1;
+	return ip->duration;
 }
 
 sample_format_t ip_get_sf(struct input_plugin *ip)
