@@ -1,7 +1,7 @@
 /*
  * Copyright 2004-2006 Timo Hirvonen
  *
- * keys.[ch] by Frank Terbeck <frank.terbeck@rwth-aachen.de>
+ * keys.[ch] by Frank Terbeck <ft@bewatermyfriend.de>
  * heavily modified by Timo Hirvonen
  *
  * This program is free software; you can redistribute it and/or
@@ -20,10 +20,11 @@
  * 02111-1307, USA.
  */
 
-#include <keys.h>
-#include <ui_curses.h>
-#include <command_mode.h>
-#include <xmalloc.h>
+#include "keys.h"
+#include "help.h"
+#include "ui_curses.h"
+#include "command_mode.h"
+#include "xmalloc.h"
 
 #if defined(__sun__)
 #include <ncurses.h>
@@ -38,6 +39,7 @@ const char * const key_context_names[NR_CTXS + 1] = {
 	"library",
 	"playlist",
 	"queue",
+	"settings",
 	NULL
 };
 
@@ -49,7 +51,8 @@ static const enum key_context view_to_context[] = {
 	CTX_PLAYLIST,
 	CTX_QUEUE,
 	CTX_BROWSER,
-	CTX_FILTERS
+	CTX_FILTERS,
+	CTX_SETTINGS,
 };
 
 #define KEY_IS_CHAR -255
@@ -472,6 +475,7 @@ int key_bind(const char *context, const char *key, const char *cmd, int force)
 {
 	const struct key *k;
 	struct binding *b, *ptr, *prev;
+	struct command *command;
 	int c, size;
 
 	c = find_context(context);
@@ -496,6 +500,7 @@ int key_bind(const char *context, const char *key, const char *cmd, int force)
 
 	b = xmalloc(sizeof(struct binding) + size);
 	b->key = k;
+	b->ctx = c;
 	memcpy(b->cmd, cmd, size);
 
 	/* insert keeping sorted by key */
@@ -513,6 +518,10 @@ int key_bind(const char *context, const char *key, const char *cmd, int force)
 	} else {
 		key_bindings[c] = b;
 	}
+	command = get_command(cmd);
+	if (command && !command->bc++)
+		help_remove_unbound(command);
+	help_add_bound(b);
 	return 0;
 bound:
 	error_msg("key %s already bound in context %s", key, key_context_names[c]);
@@ -524,6 +533,7 @@ int key_unbind(const char *context, const char *key, int force)
 	enum key_context c;
 	const struct key *k;
 	struct binding *b, *prev;
+	struct command *command;
 
 	c = find_context(context);
 	if (c < 0)
@@ -542,6 +552,10 @@ int key_unbind(const char *context, const char *key, int force)
 			} else {
 				key_bindings[c] = b->next;
 			}
+			command = get_command(b->cmd);
+			if (command && !--command->bc)
+				help_add_unbound(command);
+			help_remove_bound(b);
 			free(b);
 			return 0;
 		}
