@@ -2,17 +2,19 @@
  * Copyright 2005-2006 Timo Hirvonen
  */
 
-#include <prog.h>
-#include <file.h>
-#include <path.h>
-#include <xmalloc.h>
-#include "config/version.h"
+#include "prog.h"
+#include "file.h"
+#include "path.h"
+#include "xmalloc.h"
 
 #include <unistd.h>
 #include <fcntl.h>
-#include <sys/un.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/un.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <netdb.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -20,25 +22,25 @@
 
 static int sock;
 static int raw_args = 0;
+static char *passwd;
 
-static void remote_connect(const char *server)
+static void gethostbyname_failed(void)
 {
-	struct sockaddr_un addr;
+	const char *error = "Unknown error.";
 
-	sock = socket(PF_UNIX, SOCK_STREAM, 0);
-	if (sock == -1)
-		die_errno("socket");
-	addr.sun_family = AF_UNIX;
-	strncpy(addr.sun_path, server, sizeof(addr.sun_path) - 1);
-	if (connect(sock, (struct sockaddr *)&addr, sizeof(addr))) {
-		if (errno == ENOENT || errno == ECONNREFUSED) {
-			/* "cmus-remote -C" can be used to check if cmus is running */
-			if (!raw_args)
-				warn("cmus is not running\n");
-			exit(1);
-		}
-		die_errno("connect");
+	switch (h_errno) {
+	case HOST_NOT_FOUND:
+	case NO_ADDRESS:
+		error = "Host not found.";
+		break;
+	case NO_RECOVERY:
+		error = "A non-recoverable name server error.";
+		break;
+	case TRY_AGAIN:
+		error = "A temporary error occurred on an authoritative name server.";
+		break;
 	}
+	die("gethostbyname: %s\n", error);
 }
 
 static void write_line(const char *line)
@@ -59,6 +61,71 @@ static void send_cmd(const char *format, ...)
 	write_line(buf);
 }
 
+static void remote_connect(const char *address)
+{
+	union {
+		struct sockaddr sa;
+		struct sockaddr_un un;
+		struct sockaddr_in in;
+	} addr;
+	int addrlen;
+
+	if (strchr(address, '/')) {
+		if (passwd)
+			warn("password ignored for unix connections\n");
+		passwd = NULL;
+
+		addr.sa.sa_family = AF_UNIX;
+		strncpy(addr.un.sun_path, address, sizeof(addr.un.sun_path) - 1);
+
+		addrlen = sizeof(addr.un);
+	} else {
+		char *s = strchr(address, ':');
+		int port = DEFAULT_PORT;
+		struct hostent *hent;
+
+		if (!passwd)
+			die("password required for tcp/ip connection\n");
+
+		if (s) {
+			*s++ = 0;
+			port = atoi(s);
+		}
+		hent = gethostbyname(address);
+		if (!hent)
+			gethostbyname_failed();
+
+		addr.sa.sa_family = hent->h_addrtype;
+		switch (addr.sa.sa_family) {
+		case AF_INET:
+			memcpy(&addr.in.sin_addr, hent->h_addr_list[0], hent->h_length);
+			addr.in.sin_port = htons(port);
+
+			addrlen = sizeof(addr.in);
+			break;
+		default:
+			die("unsupported address type\n");
+		}
+	}
+
+	sock = socket(addr.sa.sa_family, SOCK_STREAM, 0);
+	if (sock == -1)
+		die_errno("socket");
+
+	if (connect(sock, &addr.sa, addrlen)) {
+		if (errno == ENOENT || errno == ECONNREFUSED) {
+			/* "cmus-remote -C" can be used to check if cmus is running */
+			if (!raw_args)
+				warn("cmus is not running\n");
+			exit(1);
+		}
+		die_errno("connect");
+	}
+
+	if (passwd)
+		send_cmd("%s\n", passwd);
+}
+
 static char *file_url_absolute(const char *str)
 {
 	char *absolute;
@@ -74,6 +141,7 @@ static char *file_url_absolute(const char *str)
 
 enum flags {
 	FLAG_SERVER,
+	FLAG_PASSWD,
 	FLAG_HELP,
 	FLAG_VERSION,
 
@@ -99,6 +167,7 @@ enum flags {
 
 static struct option options[NR_FLAGS + 1] = {
 	{ 0, "server", 1 },
+	{ 0, "passwd", 1 },
 	{ 0, "help", 0 },
 	{ 0, "version", 0 },
 
@@ -130,7 +199,9 @@ static const char *usage =
 "   or: %s\n"
 "Control cmus through socket.\n"
 "\n"
-"      --server SOCKET  connect using socket SOCKET instead of /tmp/cmus-$USER\n"
+"      --server ADDR    connect using ADDR instead of ~/.cmus/socket\n"
+"                       ADDR is either a UNIX socket or host[:port]\n"
+"      --passwd PASSWD  password to use for TCP/IP connection\n"
 "      --help           display this help and exit\n"
 "      --version        " VERSION "\n"
 "\n"
@@ -191,6 +262,9 @@ int main(int argc, char *argv[])
 		case FLAG_SERVER:
 			server = arg;
 			break;
+		case FLAG_PASSWD:
+			passwd = arg;
+			break;
 		case FLAG_VOLUME:
 			volume = arg;
 			nr_cmds++;
