@@ -20,7 +20,7 @@
 #ifndef _MISC_H
 #define _MISC_H
 
-#include <list.h>
+#include "list.h"
 
 extern const char *cmus_config_dir;
 extern const char *home_dir;
