@@ -36,10 +36,12 @@
 
 char *output_plugin = NULL;
 char *status_display_program = NULL;
+char *server_password;
 int auto_reshuffle = 0;
 int confirm_run = 1;
 int show_hidden = 0;
 int show_remaining_time = 0;
+int set_term_title = 1;
 int play_library = 1;
 int repeat = 0;
 int shuffle = 0;
@@ -285,6 +287,27 @@ static void set_output_plugin(unsigned int id, const char *buf)
 	}
 }
 
+static void get_passwd(unsigned int id, char *buf)
+{
+	if (server_password)
+		strcpy(buf, server_password);
+}
+
+static void set_passwd(unsigned int id, const char *buf)
+{
+	int len = strlen(buf);
+
+	if (len == 0) {
+		free(server_password);
+		server_password = NULL;
+	} else if (len < 6) {
+		error_msg("unsafe password");
+	} else {
+		free(server_password);
+		server_password = xstrdup(buf);
+	}
+}
+
 static void get_softvol_state(unsigned int id, char *buf)
 {
 	sprintf(buf, "%d %d", soft_vol_l, soft_vol_r);
@@ -388,7 +411,7 @@ static void toggle_confirm_run(unsigned int id)
 }
 
 const char * const view_names[NR_VIEWS + 1] = {
-	"tree", "sorted", "playlist", "queue", "browser", "filters", NULL
+	"tree", "sorted", "playlist", "queue", "browser", "filters", "settings", NULL
 };
 
 static void get_play_library(unsigned int id, char *buf)
@@ -528,6 +551,21 @@ static void toggle_show_remaining_time(unsigned int id)
 	update_statusline();
 }
 
+static void get_set_term_title(unsigned int id, char *buf)
+{
+	strcpy(buf, bool_names[set_term_title]);
+}
+
+static void set_set_term_title(unsigned int id, const char *buf)
+{
+	parse_bool(buf, &set_term_title);
+}
+
+static void toggle_set_term_title(unsigned int id)
+{
+	set_term_title ^= 1;
+}
+
 static void get_shuffle(unsigned int id, char *buf)
 {
 	strcpy(buf, bool_names[shuffle]);
@@ -663,12 +701,14 @@ static const struct {
 	DN(id3_default_charset)
 	DN(lib_sort)
 	DN(output_plugin)
+	DN(passwd)
 	DN(pl_sort)
 	DT(play_library)
 	DT(play_sorted)
 	DT(repeat)
 	DT(show_hidden)
 	DT(show_remaining_time)
+	DT(set_term_title)
 	DT(shuffle)
 	DT(softvol)
 	DN(softvol_state)
@@ -713,7 +753,7 @@ static const struct {
 	{ "altformat_title",	"%f" },
 	{ "altformat_trackwin",	" %f%= %d " },
 	{ "format_current",	" %a - %l - %02n. %t%= %y " },
-	{ "format_playlist",	" %a - %l - %02n. %t%= %y %d " },
+	{ "format_playlist",	" %-20a %02n. %t%= %y %d " },
 	{ "format_title",	"%a - %l - %t (%y)" },
 	{ "format_trackwin",	" %02n. %t%= %y %d " },
 
