@@ -17,15 +17,15 @@
  * 02111-1307, USA.
  */
 
-#include <player.h>
-#include <buffer.h>
-#include <input.h>
-#include <output.h>
-#include <sf.h>
-#include <utils.h>
-#include <xmalloc.h>
-#include <debug.h>
-#include <compiler.h>
+#include "player.h"
+#include "buffer.h"
+#include "input.h"
+#include "output.h"
+#include "sf.h"
+#include "utils.h"
+#include "xmalloc.h"
+#include "debug.h"
+#include "compiler.h"
 
 #include <stdlib.h>
 #include <pthread.h>
@@ -83,10 +83,10 @@ static pthread_t consumer_thread;
 static pthread_mutex_t consumer_mutex = CMUS_MUTEX_INITIALIZER;
 static int consumer_running = 1;
 static enum consumer_status consumer_status = CS_STOPPED;
-static int consumer_pos = 0;
+static unsigned int consumer_pos = 0;
 
 /* usually same as consumer_pos, sometimes less than consumer_pos */
-static int soft_vol_pos;
+static unsigned int soft_vol_pos;
 
 /* locking {{{ */
 
@@ -160,16 +160,17 @@ static inline void scale_sample(signed short *buf, int i, int vol)
 	}
 }
 
-static void soft_vol_scale(char *buffer, int *countp)
+static void soft_vol_scale(char *buffer, unsigned int *countp)
 {
 	signed short *buf;
-	int count = *countp;
+	unsigned int count = *countp;
 	int ch, bits, l, r, i;
 
+	BUG_ON(soft_vol_pos < consumer_pos);
+
 	if (consumer_pos != soft_vol_pos) {
-		int offs = soft_vol_pos - consumer_pos;
+		unsigned int offs = soft_vol_pos - consumer_pos;
 
-		BUG_ON(offs < 0);
 		if (offs >= count)
 			return;
 		buffer += offs;
@@ -199,7 +200,7 @@ static void soft_vol_scale(char *buffer, int *countp)
 	}
 }
 
-static inline int buffer_second_size(void)
+static inline unsigned int buffer_second_size(void)
 {
 	return sf_get_second_size(buffer_sf);
 }
@@ -342,10 +343,9 @@ static void __producer_buffer_fill_update(void)
  */
 static void __consumer_position_update(void)
 {
-	static int old_pos = -1;
-	int pos;
+	static unsigned int old_pos = -1;
+	unsigned int pos = 0;
 	
-	pos = 0;
 	if (consumer_status == CS_PLAYING || consumer_status == CS_PAUSED)
 		pos = consumer_pos / buffer_second_size();
 	if (pos != old_pos) {
@@ -364,7 +364,7 @@ static void __consumer_position_update(void)
  */
 static void __player_status_changed(void)
 {
-	int pos = 0;
+	unsigned int pos = 0;
 
 /* 	d_print("\n"); */
 	if (consumer_status == CS_PLAYING || consumer_status == CS_PAUSED)
@@ -416,9 +416,8 @@ static void __prebuffer(void)
 			if (nr_read == -1 && errno == EAGAIN)
 				continue;
 			player_ip_error(nr_read, "reading file %s", ip_get_filename(ip));
-			ip_delete(ip);
-			producer_status = PS_UNLOADED;
-			return;
+			/* ip_read sets eof */
+			nr_read = 0;
 		}
 		if (ip_metadata_changed(ip))
 			metadata_changed();
@@ -763,12 +762,13 @@ static void *producer_loop(void *arg)
 				if (nr_read != -1 || errno != EAGAIN) {
 					player_ip_error(nr_read, "reading file %s",
 							ip_get_filename(ip));
-					ip_delete(ip);
-					producer_status = PS_UNLOADED;
+					/* ip_read sets eof */
+					nr_read = 0;
+				} else {
+					producer_unlock();
+					ms_sleep(50);
+					break;
 				}
-				producer_unlock();
-				ms_sleep(50);
-				break;
 			}
 			if (ip_metadata_changed(ip))
 				metadata_changed();
@@ -904,6 +904,21 @@ void player_play(void)
 void player_pause(void)
 {
 	player_lock();
+
+	if (consumer_status == CS_STOPPED) {
+		__producer_play();
+		if (producer_status == PS_PLAYING) {
+			__consumer_play();
+			if (consumer_status != CS_PLAYING)
+				__producer_stop();
+		}
+		__player_status_changed();
+		if (consumer_status == CS_PLAYING)
+			__prebuffer();
+		player_unlock();
+		return;
+	}
+
 	if (ip && ip_is_remote(ip)) {
 		/* pausing not allowed */
 		player_unlock();
