@@ -17,12 +17,12 @@
  * 02111-1307, USA.
  */
 
-#include <tabexp_file.h>
-#include <tabexp.h>
-#include <load_dir.h>
-#include <misc.h>
-#include <xmalloc.h>
-#include <xstrjoin.h>
+#include "tabexp_file.h"
+#include "tabexp.h"
+#include "load_dir.h"
+#include "misc.h"
+#include "xmalloc.h"
+#include "xstrjoin.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -121,6 +121,7 @@ static void tabexp_load_dir(const char *dirname, const char *start,
 		}
 		ptr_array_add(&array, str);
 	}
+	dir_close(&dir);
 	if (array.count) {
 		ptr_array_sort(&array, strptrcmp);
 		ptr_array_plug(&array);
