@@ -17,31 +17,31 @@
  * 02111-1307, USA.
  */
 
-#include <ui_curses.h>
-#include <cmdline.h>
-#include <search_mode.h>
-#include <command_mode.h>
-#include <options.h>
-#include <play_queue.h>
-#include <browser.h>
-#include <filters.h>
-#include <cmus.h>
-#include <player.h>
-#include <utils.h>
-#include <lib.h>
-#include <pl.h>
-#include <xmalloc.h>
-#include <xstrjoin.h>
-#include <window.h>
-#include <format_print.h>
-#include <misc.h>
-#include <prog.h>
-#include <uchar.h>
-#include <spawn.h>
-#include <server.h>
-#include <keys.h>
-#include <debug.h>
-#include "config/version.h"
+#include "ui_curses.h"
+#include "cmdline.h"
+#include "search_mode.h"
+#include "command_mode.h"
+#include "options.h"
+#include "play_queue.h"
+#include "browser.h"
+#include "filters.h"
+#include "cmus.h"
+#include "player.h"
+#include "utils.h"
+#include "lib.h"
+#include "pl.h"
+#include "xmalloc.h"
+#include "xstrjoin.h"
+#include "window.h"
+#include "format_print.h"
+#include "misc.h"
+#include "prog.h"
+#include "uchar.h"
+#include "spawn.h"
+#include "server.h"
+#include "keys.h"
+#include "debug.h"
+#include "help.h"
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -100,7 +100,6 @@ static int msg_is_error;
 static int error_count = 0;
 
 static char *server_address = NULL;
-static int remote_socket = -1;
 
 static char *charset = NULL;
 static char print_buffer[512];
@@ -122,6 +121,9 @@ static int track_win_x = 0;
 static int track_win_y = 0;
 static int track_win_w = 0;
 
+static int cursor_x;
+static int cursor_y;
+
 enum {
 	CURSED_WIN,
 	CURSED_WIN_CUR,
@@ -424,6 +426,11 @@ static void print_tree(struct window *win, int row, struct iter *iter)
 	active = lib_cur_win == lib_tree_win;
 	bkgdset(pairs[(active << 2) | (selected << 1) | current]);
 
+	if (active && selected) {
+		cursor_x = 0;
+		cursor_y = 1 + row;
+	}
+
 	pos = 0;
 	print_buffer[pos++] = ' ';
 	str = artist->name;
@@ -432,8 +439,6 @@ static void print_tree(struct window *win, int row, struct iter *iter)
 		print_buffer[pos++] = ' ';
 		str = album->name;
 	}
-	if (!str)
-		str = "<no name>";
 	pos += format_str(print_buffer + pos, str, tree_win_w - pos - 1);
 	print_buffer[pos++] = ' ';
 	print_buffer[pos++] = 0;
@@ -553,6 +558,11 @@ static void print_track(struct window *win, int row, struct iter *iter)
 	active = lib_cur_win == lib_track_win;
 	bkgdset(pairs[(active << 2) | (selected << 1) | current]);
 
+	if (active && selected) {
+		cursor_x = track_win_x;
+		cursor_y = 1 + row;
+	}
+
 	fill_track_fopts(track);
 
 	if (track_info_has_tag(tree_track_info(track))) {
@@ -577,6 +587,11 @@ static void print_editable(struct window *win, int row, struct iter *iter)
 	window_get_sel(win, &sel);
 	selected = iters_equal(iter, &sel);
 
+	if (selected) {
+		cursor_x = 0;
+		cursor_y = 1 + row;
+	}
+
 	active = 1;
 	if (!selected && track->marked) {
 		selected = 1;
@@ -617,6 +632,11 @@ static void print_browser(struct window *win, int row, struct iter *iter)
 		}
 	}
 
+	if (selected) {
+		cursor_x = 0;
+		cursor_y = 1 + row;
+	}
+
 	/* file name encoding == terminal encoding. no need to convert */
 	if (using_utf8) {
 		sprint(row + 1, 0, e->name, COLS);
@@ -643,6 +663,11 @@ static void print_filter(struct window *win, int row, struct iter *iter)
 	selected = iters_equal(iter, &sel);
 	bkgdset(pairs[(active << 2) | (selected << 1) | current]);
 
+	if (selected) {
+		cursor_x = 0;
+		cursor_y = 1 + row;
+	}
+
 	ch1 = ' ';
 	ch3 = ' ';
 	if (e->sel_stat != e->act_stat) {
@@ -657,6 +682,49 @@ static void print_filter(struct window *win, int row, struct iter *iter)
 	dump_print_buffer(row + 1, 0);
 }
 
+static void print_help(struct window *win, int row, struct iter *iter)
+{
+	struct iter sel;
+	int selected;
+	int pos;
+	char buf[512];
+	const struct help_entry *e = iter_to_help_entry(iter);
+	const struct cmus_opt *opt;
+
+	window_get_sel(win, &sel);
+	selected = iters_equal(iter, &sel);
+	bkgdset(pairs[selected << 1]);
+
+	if (selected) {
+		cursor_x = 0;
+		cursor_y = 1 + row;
+	}
+
+	switch (e->type) {
+	case HE_TEXT:
+		snprintf(buf, sizeof(buf), " %s", e->text);
+		break;
+	case HE_BOUND:
+		snprintf(buf, sizeof(buf), " %-8s %-14s %s",
+			key_context_names[e->binding->ctx],
+			e->binding->key->name,
+			e->binding->cmd);
+		break;
+	case HE_UNBOUND:
+		snprintf(buf, sizeof(buf), " %s", e->command->name);
+		break;
+	case HE_OPTION:
+		opt = e->option;
+		snprintf(buf, sizeof(buf), " %-29s ", opt->name);
+		opt->get(opt->id, buf + strlen(buf));
+		break;
+	}
+	pos = format_str(print_buffer, buf, COLS - 1);
+	print_buffer[pos++] = ' ';
+	print_buffer[pos] = 0;
+	dump_print_buffer(row + 1, 0);
+}
+
 static void update_window(struct window *win, int x, int y, int w, const char *title,
 		void (*print)(struct window *, int, struct iter *))
 {
@@ -796,6 +864,11 @@ static void update_filters_window(void)
 	update_window(filters_win, 0, 0, COLS, "Library Filters", print_filter);
 }
 
+static void update_help_window(void)
+{
+	update_window(help_win, 0, 0, COLS, "Settings", print_help);
+}
+
 static void draw_separator(void)
 {
 	int row;
@@ -809,6 +882,9 @@ static void draw_separator(void)
 
 static void do_update_view(int full)
 {
+	cursor_x = -1;
+	cursor_y = -1;
+
 	switch (cur_view) {
 	case TREE_VIEW:
 		editable_lock();
@@ -840,6 +916,9 @@ static void do_update_view(int full)
 	case FILTERS_VIEW:
 		update_filters_window();
 		break;
+	case HELP_VIEW:
+		update_help_window();
+		break;
 	}
 }
 
@@ -1069,6 +1148,9 @@ static const char *get_stream_title(void)
 
 static void set_title(const char *title)
 {
+	if (!set_term_title)
+		return;
+
 	if (t_ts) {
 		printf("%s%s%s", tgoto(t_ts, 0, 0), title, t_fs);
 		fflush(stdout);
@@ -1182,7 +1264,11 @@ static void post_update(void)
 		refresh();
 		curs_set(1);
 	} else {
-		move(LINES - 1, 0);
+		if (cursor_x >= 0) {
+			move(cursor_y, cursor_x);
+		} else {
+			move(LINES - 1, 0);
+		}
 		refresh();
 		curs_set(0);
 	}
@@ -1319,6 +1405,9 @@ void search_not_found(void)
 		case FILTERS_VIEW:
 			what = "Filter";
 			break;
+		case HELP_VIEW:
+			what = "Binding/command/option";
+			break;
 		}
 	} else {
 		switch (cur_view) {
@@ -1334,6 +1423,9 @@ void search_not_found(void)
 		case FILTERS_VIEW:
 			what = "Filter";
 			break;
+		case HELP_VIEW:
+			what = "Binding/command/option";
+			break;
 		}
 	}
 	info_msg("%s not found: %s", what, search_str ? : "");
@@ -1343,40 +1435,36 @@ void set_view(int view)
 {
 	if (view == cur_view)
 		return;
-	cur_view = view;
 
-	editable_lock();
+	cur_view = view;
 	switch (cur_view) {
 	case TREE_VIEW:
 		searchable = tree_searchable;
-		update_tree_window();
-		update_track_window();
-		draw_separator();
 		break;
 	case SORTED_VIEW:
 		searchable = lib_editable.searchable;
-		update_sorted_window();
 		break;
 	case PLAYLIST_VIEW:
 		searchable = pl_editable.searchable;
-		update_pl_window();
 		break;
 	case QUEUE_VIEW:
 		searchable = pq_editable.searchable;
-		update_play_queue_window();
 		break;
 	case BROWSER_VIEW:
 		searchable = browser_searchable;
-		update_browser_window();
 		break;
 	case FILTERS_VIEW:
 		searchable = filters_searchable;
-		update_filters_window();
+		break;
+	case HELP_VIEW:
+		searchable = help_searchable;
+		update_help_window();
 		break;
 	}
-	editable_unlock();
 
-	refresh();
+	curs_set(0);
+	do_update_view(1);
+	post_update();
 }
 
 void enter_command_mode(void)
@@ -1581,6 +1669,7 @@ static void update(void)
 			window_set_nr_rows(pl_editable.win, h - 1);
 			window_set_nr_rows(pq_editable.win, h - 1);
 			window_set_nr_rows(filters_win, h - 1);
+			window_set_nr_rows(help_win, h - 1);
 			window_set_nr_rows(browser_win, h - 1);
 			editable_unlock();
 			needs_title_update = 1;
@@ -1637,13 +1726,18 @@ static void update(void)
 	case FILTERS_VIEW:
 		needs_view_update += filters_win->changed;
 		break;
+	case HELP_VIEW:
+		needs_view_update += help_win->changed;
+		break;
 	}
 
 	/* total time changed? */
 	if (play_library) {
 		needs_status_update += lib_editable.win->changed;
+		lib_editable.win->changed = 0;
 	} else {
 		needs_status_update += pl_editable.win->changed;
+		lib_editable.win->changed = 0;
 	}
 
 	editable_unlock();
@@ -1741,7 +1835,7 @@ static void main_loop(void)
 {
 	int rc, fd_high;
 
-	fd_high = remote_socket;
+	fd_high = server_socket;
 	while (running) {
 		fd_set set;
 		struct timeval tv;
@@ -1750,7 +1844,7 @@ static void main_loop(void)
 
 		FD_ZERO(&set);
 		FD_SET(0, &set);
-		FD_SET(remote_socket, &set);
+		FD_SET(server_socket, &set);
 
 		/* Timeout must be so small that screen updates seem instant.
 		 * Only affects changes done in other threads (worker, player).
@@ -1771,11 +1865,11 @@ static void main_loop(void)
 			continue;
 		}
 
-		if (FD_ISSET(remote_socket, &set)) {
+		if (FD_ISSET(server_socket, &set)) {
 			/* no error msgs for cmus-remote */
 			display_errors = 0;
 
-			remote_server_serve();
+			server_serve();
 		}
 		if (FD_ISSET(0, &set)) {
 			/* diplay errors for interactive commands */
@@ -1904,7 +1998,7 @@ static void init_curses(void)
 
 static void init_all(void)
 {
-	remote_socket = remote_server_init(server_address);
+	server_init(server_address);
 
 	/* does not select output plugin */
 	player_init(&player_callbacks);
@@ -1918,6 +2012,7 @@ static void init_all(void)
 	cmus_init();
 	browser_init();
 	filters_init();
+	help_init();
 	cmdline_init();
 	commands_init();
 	search_mode_init();
@@ -1944,6 +2039,7 @@ static void init_all(void)
 		warn("Press <enter> to continue.");
 		fgets(buf, sizeof(buf), stdin);
 	}
+	help_add_all_unbound();
 
 	init_curses();
 }
@@ -1954,7 +2050,7 @@ static void exit_all(void)
 
 	options_exit();
 
-	remote_server_exit();
+	server_exit();
 	cmus_exit();
 	cmus_save(lib_for_each, lib_autosave_filename);
 	cmus_save(pl_for_each, pl_autosave_filename);
@@ -1963,6 +2059,7 @@ static void exit_all(void)
 	commands_exit();
 	search_mode_exit();
 	filters_exit();
+	help_exit();
 	browser_exit();
 }
 
@@ -1986,7 +2083,8 @@ static const char *usage =
 "Usage: %s [OPTION]...\n"
 "Curses based music player.\n"
 "\n"
-"      --listen ADDR   listen ADDR (unix socket) instead of /tmp/cmus-$USER\n"
+"      --listen ADDR   listen on ADDR instead of ~/.cmus/socket\n"
+"                      ADDR is either a UNIX socket or host[:port]\n"
 "      --plugins       list available plugins and exit\n"
 "      --help          display this help and exit\n"
 "      --version       " VERSION "\n"
